@@ -138,10 +138,10 @@ final class PhoneNumberMatcher implements Iterator<PhoneNumberMatch> {
      * closing bracket first. We limit the sets of brackets in a phone number to four.
      */
     MATCHING_BRACKETS = Pattern.compile(
-        "(?:[" + openingParens + "])?" + "(?:" + nonParens + "+" + "[" + closingParens + "])?" +
-        nonParens + "+" +
-        "(?:[" + openingParens + "]" + nonParens + "+[" + closingParens + "])" + bracketPairLimit +
-        nonParens + "*");
+        "(?:[" + openingParens + "])?" + "(?:" + nonParens + "+" + "[" + closingParens + "])?"
+        + nonParens + "+"
+        + "(?:[" + openingParens + "]" + nonParens + "+[" + closingParens + "])" + bracketPairLimit
+        + nonParens + "*");
 
     /* Limit on the number of leading (plus) characters. */
     String leadLimit = limit(0, 2);
@@ -167,9 +167,9 @@ final class PhoneNumberMatcher implements Iterator<PhoneNumberMatch> {
 
     /* Phone number pattern allowing optional punctuation. */
     PATTERN = Pattern.compile(
-        "(?:" + leadClass + punctuation + ")" + leadLimit +
-        digitSequence + "(?:" + punctuation + digitSequence + ")" + blockLimit +
-        "(?:" + PhoneNumberUtil.EXTN_PATTERNS_FOR_MATCHING + ")?",
+        "(?:" + leadClass + punctuation + ")" + leadLimit
+        + digitSequence + "(?:" + punctuation + digitSequence + ")" + blockLimit
+        + "(?:" + PhoneNumberUtil.EXTN_PATTERNS_FOR_MATCHING + ")?",
         PhoneNumberUtil.REGEX_FLAGS);
   }
 
@@ -211,16 +211,16 @@ final class PhoneNumberMatcher implements Iterator<PhoneNumberMatch> {
    * Creates a new instance. See the factory methods in {@link PhoneNumberUtil} on how to obtain a
    * new instance.
    *
-   * @param util      the phone number util to use
-   * @param text      the character sequence that we will search, null for no text
-   * @param country   the country to assume for phone numbers not written in international format
-   *                  (with a leading plus, or with the international dialing prefix of the
-   *                  specified region). May be null or "ZZ" if only numbers with a
-   *                  leading plus should be considered.
+   * @param util  the phone number util to use
+   * @param text  the character sequence that we will search, null for no text
+   * @param country  the country to assume for phone numbers not written in international format
+   *     (with a leading plus, or with the international dialing prefix of the specified region).
+   *     May be null or "ZZ" if only numbers with a leading plus should be
+   *     considered.
    * @param leniency  the leniency to use when evaluating candidate phone numbers
    * @param maxTries  the maximum number of invalid numbers to try before giving up on the text.
-   *                  This is to cover degenerate cases where the text has a lot of false positives
-   *                  in it. Must be {@code >= 0}.
+   *     This is to cover degenerate cases where the text has a lot of false positives in it. Must
+   *     be {@code >= 0}.
    */
   PhoneNumberMatcher(PhoneNumberUtil util, CharSequence text, String country, Leniency leniency,
       long maxTries) {
@@ -292,12 +292,12 @@ final class PhoneNumberMatcher implements Iterator<PhoneNumberMatch> {
       return false;
     }
     UnicodeBlock block = UnicodeBlock.of(letter);
-    return block.equals(UnicodeBlock.BASIC_LATIN) ||
-        block.equals(UnicodeBlock.LATIN_1_SUPPLEMENT) ||
-        block.equals(UnicodeBlock.LATIN_EXTENDED_A) ||
-        block.equals(UnicodeBlock.LATIN_EXTENDED_ADDITIONAL) ||
-        block.equals(UnicodeBlock.LATIN_EXTENDED_B) ||
-        block.equals(UnicodeBlock.COMBINING_DIACRITICAL_MARKS);
+    return block.equals(UnicodeBlock.BASIC_LATIN)
+        || block.equals(UnicodeBlock.LATIN_1_SUPPLEMENT)
+        || block.equals(UnicodeBlock.LATIN_EXTENDED_A)
+        || block.equals(UnicodeBlock.LATIN_EXTENDED_ADDITIONAL)
+        || block.equals(UnicodeBlock.LATIN_EXTENDED_B)
+        || block.equals(UnicodeBlock.COMBINING_DIACRITICAL_MARKS);
   }
 
   private static boolean isInvalidPunctuationSymbol(char character) {
@@ -326,15 +326,14 @@ final class PhoneNumberMatcher implements Iterator<PhoneNumberMatch> {
     }
 
     // Try to come up with a valid match given the entire candidate.
-    String rawString = candidate.toString();
-    PhoneNumberMatch match = parseAndVerify(rawString, offset);
+    PhoneNumberMatch match = parseAndVerify(candidate, offset);
     if (match != null) {
       return match;
     }
 
     // If that failed, try to find an "inner match" - there might be a phone number within this
     // candidate.
-    return extractInnerMatch(rawString, offset);
+    return extractInnerMatch(candidate, offset);
   }
 
   /**
@@ -345,9 +344,8 @@ final class PhoneNumberMatcher implements Iterator<PhoneNumberMatch> {
    * @param offset  the current offset of {@code candidate} within {@link #text}
    * @return  the match found, null if none can be found
    */
-  private PhoneNumberMatch extractInnerMatch(String candidate, int offset) {
+  private PhoneNumberMatch extractInnerMatch(CharSequence candidate, int offset) {
     for (Pattern possibleInnerMatch : INNER_MATCHES) {
-      int rangeStart = 0;
       Matcher groupMatcher = possibleInnerMatch.matcher(candidate);
       boolean isFirstMatch = true;
       while (groupMatcher.find() && maxTries > 0) {
@@ -355,8 +353,8 @@ final class PhoneNumberMatcher implements Iterator<PhoneNumberMatch> {
           // We should handle any group before this one too.
           CharSequence group = trimAfterFirstMatch(
               PhoneNumberUtil.UNWANTED_END_CHAR_PATTERN,
-              candidate.substring(0, groupMatcher.start()));
-          PhoneNumberMatch match = parseAndVerify(group.toString(), offset);
+              candidate.subSequence(0, groupMatcher.start()));
+          PhoneNumberMatch match = parseAndVerify(group, offset);
           if (match != null) {
             return match;
           }
@@ -365,7 +363,7 @@ final class PhoneNumberMatcher implements Iterator<PhoneNumberMatch> {
         }
         CharSequence group = trimAfterFirstMatch(
             PhoneNumberUtil.UNWANTED_END_CHAR_PATTERN, groupMatcher.group(1));
-        PhoneNumberMatch match = parseAndVerify(group.toString(), offset + groupMatcher.start(1));
+        PhoneNumberMatch match = parseAndVerify(group, offset + groupMatcher.start(1));
         if (match != null) {
           return match;
         }
@@ -384,7 +382,7 @@ final class PhoneNumberMatcher implements Iterator<PhoneNumberMatch> {
    * @param offset  the offset of {@code candidate} within {@link #text}
    * @return  the parsed and validated phone number match, or null
    */
-  private PhoneNumberMatch parseAndVerify(String candidate, int offset) {
+  private PhoneNumberMatch parseAndVerify(CharSequence candidate, int offset) {
     try {
       // Check the candidate doesn't contain any formatting which would indicate that it really
       // isn't a phone number.
@@ -415,25 +413,6 @@ final class PhoneNumberMatcher implements Iterator<PhoneNumberMatch> {
 
       PhoneNumber number = phoneUtil.parseAndKeepRawInput(candidate, preferredRegion);
 
-      // Check Israel * numbers: these are a special case in that they are four-digit numbers that
-      // our library supports, but they can only be dialled with a leading *. Since we don't
-      // actually store or detect the * in our phone number library, this means in practice we
-      // detect most four digit numbers as being valid for Israel. We are considering moving these
-      // numbers to ShortNumberInfo instead, in which case this problem would go away, but in the
-      // meantime we want to restrict the false matches so we only allow these numbers if they are
-      // preceded by a star. We enforce this for all leniency levels even though these numbers are
-      // technically accepted by isPossibleNumber and isValidNumber since we consider it to be a
-      // deficiency in those methods that they accept these numbers without the *.
-      // TODO: Remove this or make it significantly less hacky once we've decided how to
-      // handle these short codes going forward in ShortNumberInfo. We could use the formatting
-      // rules for instance, but that would be slower.
-      if (phoneUtil.getRegionCodeForCountryCode(number.getCountryCode()).equals("IL") &&
-          phoneUtil.getNationalSignificantNumber(number).length() == 4 &&
-          (offset == 0 || (offset > 0 && text.charAt(offset - 1) != '*'))) {
-        // No match.
-        return null;
-      }
-
       if (leniency.verify(number, candidate, phoneUtil)) {
         // We used parseAndKeepRawInput to create this number, but for now we don't return the extra
         // values parsed. TODO: stop clearing all values here and switch all users over
@@ -441,7 +420,7 @@ final class PhoneNumberMatcher implements Iterator<PhoneNumberMatch> {
         number.clearCountryCodeSource();
         number.clearRawInput();
         number.clearPreferredDomesticCarrierCode();
-        return new PhoneNumberMatch(offset, candidate, number);
+        return new PhoneNumberMatch(offset, candidate.toString(), number);
       }
     } catch (NumberParseException e) {
       // ignore and continue
@@ -496,8 +475,8 @@ final class PhoneNumberMatcher implements Iterator<PhoneNumberMatch> {
         // as we do not need to distinguish between different countries with the same country
         // calling code and this is faster.
         String region = util.getRegionCodeForCountryCode(number.getCountryCode());
-        if (util.getNddPrefixForRegion(region, true) != null &&
-            Character.isDigit(normalizedCandidate.charAt(fromIndex))) {
+        if (util.getNddPrefixForRegion(region, true) != null
+            && Character.isDigit(normalizedCandidate.charAt(fromIndex))) {
           // This means there is no formatting symbol after the NDC. In this case, we only
           // accept the number if there is no formatting symbol at all in the number, except
           // for extensions. This is only important for countries with national prefixes.
@@ -525,8 +504,8 @@ final class PhoneNumberMatcher implements Iterator<PhoneNumberMatch> {
     // First we check if the national significant number is formatted as a block.
     // We use contains and not equals, since the national significant number may be present with
     // a prefix such as a national number prefix, or the country code itself.
-    if (candidateGroups.length == 1 ||
-        candidateGroups[candidateNumberGroupIndex].contains(
+    if (candidateGroups.length == 1
+        || candidateGroups[candidateNumberGroupIndex].contains(
             util.getNationalSignificantNumber(number))) {
       return true;
     }
@@ -542,8 +521,8 @@ final class PhoneNumberMatcher implements Iterator<PhoneNumberMatch> {
     }
     // Now check the first group. There may be a national prefix at the start, so we only check
     // that the candidate group ends with the formatted number group.
-    return (candidateNumberGroupIndex >= 0 &&
-            candidateGroups[candidateNumberGroupIndex].endsWith(formattedNumberGroups[0]));
+    return (candidateNumberGroupIndex >= 0
+        && candidateGroups[candidateNumberGroupIndex].endsWith(formattedNumberGroups[0]));
   }
 
   /**
@@ -573,7 +552,8 @@ final class PhoneNumberMatcher implements Iterator<PhoneNumberMatch> {
   }
 
   static boolean checkNumberGroupingIsValid(
-      PhoneNumber number, String candidate, PhoneNumberUtil util, NumberGroupingChecker checker) {
+      PhoneNumber number, CharSequence candidate, PhoneNumberUtil util,
+      NumberGroupingChecker checker) {
     // TODO: Evaluate how this works for other locales (testing has been limited to NANPA regions)
     // and optimise if necessary.
     StringBuilder normalizedCandidate =
@@ -611,10 +591,10 @@ final class PhoneNumberMatcher implements Iterator<PhoneNumberMatch> {
 
     // If the first slash is after the country calling code, this is permitted.
     boolean candidateHasCountryCode =
-        (number.getCountryCodeSource() == CountryCodeSource.FROM_NUMBER_WITH_PLUS_SIGN ||
-         number.getCountryCodeSource() == CountryCodeSource.FROM_NUMBER_WITHOUT_PLUS_SIGN);
-    if (candidateHasCountryCode &&
-        PhoneNumberUtil.normalizeDigitsOnly(candidate.substring(0, firstSlashInBodyIndex))
+        (number.getCountryCodeSource() == CountryCodeSource.FROM_NUMBER_WITH_PLUS_SIGN
+         || number.getCountryCodeSource() == CountryCodeSource.FROM_NUMBER_WITHOUT_PLUS_SIGN);
+    if (candidateHasCountryCode
+        && PhoneNumberUtil.normalizeDigitsOnly(candidate.substring(0, firstSlashInBodyIndex))
             .equals(Integer.toString(number.getCountryCode()))) {
       // Any more slashes and this is illegal.
       return candidate.substring(secondSlashInBodyIndex + 1).contains("/");
@@ -670,7 +650,7 @@ final class PhoneNumberMatcher implements Iterator<PhoneNumberMatch> {
     // To do this, we check that a national prefix formatting rule was present and that it wasn't
     // just the first-group symbol ($1) with punctuation.
     if ((formatRule != null) && formatRule.getNationalPrefixFormattingRule().length() > 0) {
-      if (formatRule.isNationalPrefixOptionalWhenFormatting()) {
+      if (formatRule.getNationalPrefixOptionalWhenFormatting()) {
         // The national-prefix is optional in these cases, so we don't need to check if it was
         // present.
         return true;
@@ -690,6 +670,7 @@ final class PhoneNumberMatcher implements Iterator<PhoneNumberMatch> {
     return true;
   }
 
+  @Override
   public boolean hasNext() {
     if (state == State.NOT_READY) {
       lastMatch = find(searchIndex);
@@ -703,6 +684,7 @@ final class PhoneNumberMatcher implements Iterator<PhoneNumberMatch> {
     return state == State.READY;
   }
 
+  @Override
   public PhoneNumberMatch next() {
     // Check the state and find the next match as a side-effect if necessary.
     if (!hasNext()) {
@@ -719,6 +701,7 @@ final class PhoneNumberMatcher implements Iterator<PhoneNumberMatch> {
   /**
    * Always throws {@link UnsupportedOperationException} as removal is not supported.
    */
+  @Override
   public void remove() {
     throw new UnsupportedOperationException();
   }
